@@ -1,5 +1,4 @@
 
-from typing import List
 from orchestrator.orchestrator_http import OrchestratorHTTP
 import requests
 import json
@@ -7,7 +6,8 @@ from urllib.parse import urlencode
 from orchestrator.orchestrator_folder import Folder
 from orchestrator.orchestrator_library import Library
 from orchestrator.orchestrator_process import Process
-
+from orchestrator.orchestrator_machine import Machine
+from pprint import pprint
 
 __all__ = ["Orchestrator"]
 
@@ -64,10 +64,10 @@ class Orchestrator(OrchestratorHTTP):
             self.folder_id = folder_id
             self.base_url = f"{self.cloud_url}/{self.tenant_name}/JTBOT/odata"
         if session:
-            print("session set")
+            # print("session set")
             self.session = session
         else:
-            print("session not set")
+            # print("session not set")
             self.session = requests.Session()
 
     def __str__(self):
@@ -91,7 +91,7 @@ class Orchestrator(OrchestratorHTTP):
             url = f"{self.base_url}{endpoint}"
         data = self._get(url)
         filt_data = data['value']
-        return [Folder(self.client_id, self.refresh_token, self.tenant_name, self.session, folder["DisplayName"], folder["Id"]) for folder in filt_data]
+        return [Folder(self.client_id, self.refresh_token, self.tenant_name, self.session, folder["DisplayName"], folder["Id"], access_token=self.access_token) for folder in filt_data]
 
     def get_folder_ids(self, options=None):
         """
@@ -118,9 +118,9 @@ class Orchestrator(OrchestratorHTTP):
         @returns: a Folder object with the specified folder id
         """
         ids = self.get_folder_ids()
-        self.folder_id = folder_id
-        folder_name = ids[folder_id]
-        return Folder(client_id=self.client_id, refresh_token=self.refresh_token, tenant_name=self.tenant_name,  session=self.session, folder_name=folder_name, folder_id=int(folder_id))
+        self.folder_id = int(folder_id)
+        folder_name = ids[int(folder_id)]
+        return Folder(client_id=self.client_id, refresh_token=self.refresh_token, tenant_name=self.tenant_name,  session=self.session, folder_name=folder_name, folder_id=int(folder_id), access_token=self.access_token)
 
     def get_folder_by_name(self, folder_name):
         """
@@ -130,70 +130,20 @@ class Orchestrator(OrchestratorHTTP):
             ============
             @returns: a Folder object with the specified folder_name
         """
-        pass
+        folders = self.get_folders()
+        for folder in folders:
+            if folder.name == folder_name:
+                return folder
+        raise Exception(f"Folder {folder_name} not found")
 
-    def usernames(self, options=None):
-        """
-            No se por que no va
-        """
-        endpoint = "/Sessions"
-        uipath_svc = "/UiPath.Server,Configuration.OData.GetUsernames"
-        if options:
-            query_params = urlencode(options)
-            url = f"{self.base_url}{endpoint}{uipath_svc}?{query_params}"
-        else:
-            url = f"{self.base_url}{endpoint}{uipath_svc}"
-        return self._get(url)
-
-    def get_processes(self, options=None):
-        """
-        Gets all the processes of a given organization
-
-        @options: a dictionary of odata filtering options
-        ========
-        @returns: a list of Processes of the given organization
-        """
-        endpoint = "/Processes"
+    def permissions(self, options=None):
+        endpoint = "/Permissions"
         if options:
             query_params = urlencode(options)
             url = f"{self.base_url}{endpoint}?{query_params}"
         else:
             url = f"{self.base_url}{endpoint}"
-        processes = self._get(url)["value"]
-        return [Process(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.session, process["Id"], process["Title"], process["Version"], process["Key"]) for process in processes]
-
-    def get_processes_keys(self, options=None):
-        """
-            Returns a dictionary with the processes keys
-
-            @options: dictionary of odata filtering options
-            ========
-            @returns: a dictionary where the keys are the process'
-            key and the values the process' title of the processes 
-            in the given organization
-        """
-        processes = self.get_processes(options=options)
-        ids = {}
-        for process in processes:
-            ids.update({process.key: process.title})
-        return ids
-
-    def get_process_by_key(self, process_key):
-        """
-        Returns a single process by is key
-
-        @process_key: the key of the process 
-        ============
-        @returns: a Process object with the specified process key 
-        """
-        query_param = urlencode({
-            "$filter": f"Key eq '{process_key}"
-        })
-        endpoint = "/Processes"
-        url = f"{self.base_url}{endpoint}?{query_param}"
-        process = self._get(url)["value"][0]
-        return Process(self.client_id, self.refresh_token, self.tenant_name, self.folder_id,
-                       self.session, process["Id"], process["Title"], process["Version"], process["Key"])
+        return self._get(url)
 
     def get_libraries(self, options=None):
         """
@@ -210,4 +160,42 @@ class Orchestrator(OrchestratorHTTP):
         else:
             url = f"{self.base_url}{endpoint}"
         libraries = self._get(url)["value"]
-        return [Library(self.client_id, self.refresh_token, self.tenant_name, self.session,  lib["Key"], lib["Id"], lib["Title"], self.folder_id) for lib in libraries]
+        return [Library(self.client_id, self.refresh_token, self.tenant_name, self.session,  lib["Key"], lib["Id"], lib["Title"], self.folder_id, access_token=self.access_token) for lib in libraries]
+
+    def get_machines(self, options=None):
+        """
+        Gets all the machines of a given organization 
+        """
+        endpoint = "/Machines"
+        if options:
+            query_params = urlencode(options)
+            url = f"{self.base_url}{endpoint}?{query_params}"
+        else:
+            url = f"{self.base_url}{endpoint}"
+        data = self._get(url)["value"]
+        return [Machine(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.session, machine["Id"], machine["Key"], machine["Name"], self.access_token) for machine in data]
+
+    def get_machine_ids(self, options=None):
+        """
+        Returns a dictionary of the machine keys and their
+        names 
+
+        @options: dictionary of odata filtering options 
+        """
+        machines = self.get_machines(options)
+        ids = {}
+        for machine in machines:
+            ids.update({machine.id: machine.name})
+        return ids
+
+    def get_machine_by_id(self, machine_id):
+        """
+        Returns a single machine by its id 
+
+        @machine_id: the id of the machine 
+        """
+
+        endpoint = f"/Machines({machine_id})"
+        url = f"{self.base_url}{endpoint}"
+        machine = self._get(url)
+        return Machine(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.session, int(machine_id), machine["Key"], machine["Name"], access_token=self.access_token)
