@@ -1,11 +1,14 @@
+import datetime
 import logging
 from pprint import pprint
 from uuid import uuid4
 import uuid
+import json
 from orchestrator.exceptions import OrchestratorMissingParam
 from orchestrator.orchestrator_http import OrchestratorHTTP
 import requests
 from urllib.parse import urlencode
+
 from orchestrator.orchestrator_queue_item import QueueItem
 
 __all__ = ["Queue"]
@@ -14,12 +17,12 @@ __all__ = ["Queue"]
 class Queue(OrchestratorHTTP):
     classes = ["Comments", "Status", "Reference", "SpecificContent"]
     """
-    Constructor. 
+    Constructor.
 
-    @client_id: the client id 
-    @refresh_token: a refresh token  
+    @client_id: the client id
+    @refresh_token: a refresh token
     @tenant_name: account's logical name
-    @folder_id: the folder id 
+    @folder_id: the folder id
     @folder_name: the folder name
     @session: a session object (options)
     @queue_name: the queue name
@@ -32,6 +35,7 @@ class Queue(OrchestratorHTTP):
             raise OrchestratorMissingParam(value="queue_id",
                                            message="Required parameter(s) missing: queue_id")
         self.id = queue_id
+        self.client_id = client_id
         self.name = queue_name
         self.folder_name = folder_name
         self.folder_id = folder_id
@@ -52,7 +56,7 @@ class Queue(OrchestratorHTTP):
             Returns information about the queue
 
             @returns: dictionary with more in depth information
-            about the queue 
+            about the queue
         """
         endpoint = f"/QueueDefinitions({self.id})"
         url = f"{self.base_url}{endpoint}"
@@ -65,7 +69,7 @@ class Queue(OrchestratorHTTP):
 
             @machine_identifier: the machine's unique identifier
             @specific_content: the specific content of the transaction
-            @reference: a reference from the specific content 
+            @reference: a reference from the specific content
             @fields: a dictionary of additional fields to be added to the specific content
         """
         ran_uuid = str(uuid4())
@@ -84,7 +88,7 @@ class Queue(OrchestratorHTTP):
                 ref = ""
                 for reference in references:
                     value = format_body_start["transactionData"]["SpecificContent"][reference]
-                    ref += value+separator
+                    ref += str(value)+separator
                 format_body_start["transactionData"]["Reference"] = f"{ref[:-1]}#{batch_id}"
             except KeyError as err:
                 if reference in err.args:
@@ -128,7 +132,10 @@ class Queue(OrchestratorHTTP):
             ========
             @returns: an Item object with the specified item id
         """
-        return QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, self.session, item_id, access_token=self.access_token)
+        endpoint = f"/QueueItems({item_id})"
+        url = f"{self.base_url}{endpoint}"
+        data = self._get(url)
+        return QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, self.session, item_id, content=data["SpecificContent"], reference=data["Reference"], status=data["Status"], access_token=self.access_token)
 
     def get_queue_items(self, options=None):
         """
@@ -139,23 +146,63 @@ class Queue(OrchestratorHTTP):
             @returns: a list of QueueItem objects of the given queue (Maximum number of results: 1000)
         """
         endpoint = "/QueueItems"
+        odata_filter = {}
         if options and ("$filter" in options):
-            print("true")
-            odata_filter = {"$filter": f"QueueDefinitionId eq {self.id} and {options['$filter']}"}
+            # print(options["$filter"])
+            odata_filter = {"$filter": f"{options['$filter']} and QueueDefinitionId eq {self.id}"}
+        elif not options:
+            odata_filter = {}
         else:
-            odata_filter = {"$Filter": f"QueueDefinitionId eq {self.id}"}
-        # if options:
-        #     odata_filter.update(options)
+            odata_filter = {"$filter": f"QueueDefinitionId eq {self.id} and Status in ('Failed', 'Retried', 'Successful')"}
+        # print(odata_filter)
+        if options is not None:
+            for k, v in options.items():
+                odata_filter.update({k: v})
+        # print(odata_filter)
+
         query_params = urlencode(odata_filter)
         url = f"{self.base_url}{endpoint}?{query_params}"
         data = self._get(url)
+        # pprint(odata_filter)
         # pprint(data)
+        # pprint(f"ODATA COUNT - -> {data['@odata.count']}")
         filt_data = data['value']
-        return [QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, session=self.session, item_id=item["Id"], content=item["SpecificContent"], reference=item["Reference"], access_token=self.access_token) for item in filt_data]
+        if len(odata_filter) < 2:
+            return [QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, session=self.session, item_id=item["Id"], content=item["SpecificContent"], reference=item["Reference"], status=item["Status"], access_token=self.access_token) for item in filt_data]
+        else:
+            return filt_data
+
+    def filter_by_reference(self, reference, num_days=2):
+        """
+        Returns a list of references of items which 
+        have status Failed, Retried or Successful from the past number of days
+
+        :param num_days - number of days to query the queue from (default: 2) 
+        """
+        tod = datetime.datetime.now()
+        filt_options = {
+            "$select": f"SpecificContent/{reference}, CreationTime, Status"
+        }
+        items = self.get_queue_items(options=filt_options)
+        try:
+            aux_list = [[i["SpecificContent"][reference], i["CreationTime"], i["Status"]] for i in items]
+            print(f"Length of aux_list: {len(aux_list)}")
+            for i, item in enumerate(aux_list):
+                creation_date = item[1]
+                try:
+                    diff = tod - datetime.timedelta(days=datetime.datetime.strptime(creation_date, '%Y-%m-%dT%H:%M:%S.%fZ').day)
+                except ValueError:
+                    diff = tod - datetime.timedelta(days=datetime.datetime.strptime(creation_date, '%Y-%m-%dT%H:%M:%SZ').day)
+
+                if diff.day > num_days:
+                    aux_list.pop(i)
+            return aux_list
+        except KeyError as err:
+            raise err
 
     def get_queue_items_by_status(self, status):
         """
-        Returns a list of QueueItems with the status 
+        Returns a list of QueueItems with the status
         as indicated in the argument.
         """
         endpoint = "/QueueItems"
@@ -164,7 +211,7 @@ class Queue(OrchestratorHTTP):
         url = f"{self.base_url}{endpoint}?{query_params}"
         data = self._get(url)
         filt_data = data['value']
-        return [QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, session=self.session, item_id=item["Id"], content=item["SpecificContent"], reference=item["Reference"], access_token=self.access_token) for item in filt_data]
+        return [QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, session=self.session, item_id=item["Id"], content=item["SpecificContent"], reference=item["Reference"], status=item["Status"], access_token=self.access_token) for item in filt_data]
 
     def _get_sp_contents(self, options=None):
         items = self.get_queue_items(options=options)
@@ -182,14 +229,16 @@ class Queue(OrchestratorHTTP):
 
     def check_duplicate(self, reference):
         """
-        Given a queue reference, it checks whether a given queue 
-        has already appeared in the queue once.
+        Given a queue reference, it checks whether a given queue
+        has already appeared in the queue once. It first gets the items 
+        of the queue which have status Succesful or Failed as a list of 
+
 
         Parameters:
 
             - `param` reference: the reference or part of it of the new queue item
 
-        Returns: 
+        Returns:
             If a reference is found in the queue with the given status, it returns the first
             item whose reference matches the one indicated as an argument. Otherwise it returns
             False.
@@ -203,18 +252,18 @@ class Queue(OrchestratorHTTP):
 
     def get_queue_items_ids(self, options=None):
         """
-            Returns a list of dictionaries with the queue 
-            item ids 
+            Returns a list of dictionaries with the queue
+            item ids
 
-            @options: dictionary of odata filtering options 
+            @options: dictionary of odata filtering options
             ========
             @returns: a dictionary where the keys are the queue item
             ids of the given queue and the values the queue name
         """
         items = self.get_queue_items(options)
-        ids = {}
+        ids = []
         for item in items:
-            ids.update({item.id: item.queue_name})
+            ids.append({item.id: item.queue_name})
         return ids
 
     def add_queue_item(self, specific_content=None, priority="Low"):
@@ -244,6 +293,7 @@ class Queue(OrchestratorHTTP):
 
     def _format_specific_content(self, sp_content, reference=None, priority="Low", progress="New", batch_id=None):
         ran_uuid = str(uuid4())
+
         if reference:
             try:
                 ref_uuid = {"Reference": f"{sp_content[reference]}#{batch_id}"}
@@ -274,12 +324,34 @@ class Queue(OrchestratorHTTP):
             formatted_sp_content.update(ref_uuid)
             return formatted_sp_content
 
+    def _format_dataframe(self, df):
+        specific_contents = []
+        for i in df.index:
+            specific_contents.append(df.iloc[i].to_json())
+        # pprint(specific_contents)
+        return specific_contents
+
+    def bulk_dataframe(self, df, priority="Low", progress="New", reference=None):
+        """Adds a dataframe of items to a given queue"""
+        endpoint = "/Queues"
+        uipath_svc = "/UiPathODataSvc.BulkAddQueueItems"
+        url = f"{self.base_url}{endpoint}{uipath_svc}"
+        sp_contents = self._format_dataframe(df=df)
+        batch_id = str(uuid4())
+        format_body_queue = {
+            "commitType": "StopOnFirstFailure",
+            "queueName": self.name,
+            "queueItems": [self._format_specific_content(sp_content=json.loads(sp_content), reference=reference, priority=priority, progress=progress, batch_id=batch_id) for sp_content in sp_contents]
+        }
+        # pprint(format_body_queue)
+        return self._post(url, body=format_body_queue)
+
     def bulk_create_items(self, specific_contents=None, priority="Low", progress="New", reference=None):
         """Adds a list of items for a given queue
             @param specific_content: dictionary of key value pairs. It does not
-            admit nested dictionaries. If you want to be able to pass a dictionary 
-            as a key value pair inside the specific content attribute, you need to 
-                                json.dumps(dict) 
+            admit nested dictionaries. If you want to be able to pass a dictionary
+            as a key value pair inside the specific content attribute, you need to
+                                json.dumps(dict)
             first for it to work.
             @priority: sets up the priority (default: Low)
             @progress: sets up the progress bar (default: New)
@@ -293,6 +365,7 @@ class Queue(OrchestratorHTTP):
         """
         endpoint = "/Queues"
         uipath_svc = "/UiPathODataSvc.BulkAddQueueItems"
+        print(type(specific_contents[0]))
         url = f"{self.base_url}{endpoint}{uipath_svc}"
         if not specific_contents:
             raise OrchestratorMissingParam(value="specific_contents", message="specific contents cannot be null")
@@ -301,13 +374,13 @@ class Queue(OrchestratorHTTP):
         format_body_queue = {
             "commitType": "StopOnFirstFailure",
             "queueName": self.name,
-            "queueItems": [self._format_specific_content(queue_name=self.name, sp_content=sp_content, reference=reference, priority=priority, progress=progress, batch_id=batch_id) for sp_content in specific_contents]
+            "queueItems": [self._format_specific_content(sp_content=sp_content, reference=reference, priority=priority, progress=progress, batch_id=batch_id) for sp_content in specific_contents]
         }
         # pprint(format_body_queue)
         return self._post(url, body=format_body_queue)
 
     def edit_queue(self, name=None, description=None):
-        """Edits the queue with a new name and a new 
+        """Edits the queue with a new name and a new
         descriptions
 
         @name: the new name of the queue
